(**
 * Image tilize,
 *
 * contains functions to split images in smaller parts and use
 * pipelining mechanism to process images quicker
 *
 * Code should be compiled successfully under Debian Wheezy
 * with following line:
 *
 *   ocamlopt.opt -I +camlimages camlimages_core.cmxa unix.cmxa
 *      graphics.cmxa camlimages_graphics.cmxa
 *      camlimages_all.cmxa example.ml -o example
 *
 * It depends on: camlimages and standard ocaml library
 * (on Debian Wheezy: libcamlimages-ocaml-dev, libcamlimages-ocaml,
 * ocaml-native-compilers)
 *  
 * Code is distributed under the terms of the GNU General Public License 
 * (v2.0 or higher), described in file LICENSE.
 *                                                                    
 * (c) 2015-2023 Andreas Romeyke
 * E-Mail: ocaml@andreas-romeyke.de
 *
 *)

(*
open Image_tilize
open Images
open OImages
open Rgb24
*)

(** helper to save new image 
 * @param outfile name of outfile image
 * @param img image to save
 *)
let save_image_as outfile img = img#save outfile None []

(** helper tp load image
 * @param infile name of infile image
 * @return image
 *)
let load infile = OImages.rgb24 (OImages.load infile [])

(** main routine *)
let _ =
  try
    let infile = Sys.argv.(1) in
    let outfile_prefix = Sys.argv.(2) in
    let img = load infile in
    let dims = Printf.sprintf " %ix%i" img#width img#height in
    Printf.printf "dims: %s\n" dims;
    let pipeline = Image_tilize.pipelinize 100 200 10 20 img in

    List.iteri
      (fun i imgi ->
        save_image_as (Printf.sprintf "%s%i.png" outfile_prefix i) imgi)
      pipeline.Image_tilize.tiles;

    let img' = Image_tilize.unpipelinize pipeline in
    save_image_as (outfile_prefix ^ ".debug_out.png") img'
  with
  | Stack_overflow ->
      Printf.printf
        "A stackoverflow occured, please increase stacksize with 'ulimit -s \
         65535'\n"
  | Invalid_argument s -> Printf.printf "\n%s\n" s
